# coding: utf-8

"""
    Lightly API

    Lightly.ai enables you to do self-supervised learning in an easy and intuitive way. The lightly.ai OpenAPI spec defines how one can interact with our REST API to unleash the full potential of lightly.ai  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@lightly.ai
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from lightly.openapi_generated.swagger_client.configuration import Configuration


class SelectionConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'n_samples': 'int',
        'proportion_samples': 'float',
        'strategies': 'list[SelectionConfigEntry]'
    }

    attribute_map = {
        'n_samples': 'nSamples',
        'proportion_samples': 'proportionSamples',
        'strategies': 'strategies'
    }

    def __init__(self, n_samples=None, proportion_samples=None, strategies=None, _configuration=None):  # noqa: E501
        """SelectionConfig - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._n_samples = None
        self._proportion_samples = None
        self._strategies = None
        self.discriminator = None

        if n_samples is not None:
            self.n_samples = n_samples
        if proportion_samples is not None:
            self.proportion_samples = proportion_samples
        self.strategies = strategies

    @property
    def n_samples(self):
        """Gets the n_samples of this SelectionConfig.  # noqa: E501


        :return: The n_samples of this SelectionConfig.  # noqa: E501
        :rtype: int
        """
        return self._n_samples

    @n_samples.setter
    def n_samples(self, n_samples):
        """Sets the n_samples of this SelectionConfig.


        :param n_samples: The n_samples of this SelectionConfig.  # noqa: E501
        :type: int
        """

        self._n_samples = n_samples

    @property
    def proportion_samples(self):
        """Gets the proportion_samples of this SelectionConfig.  # noqa: E501


        :return: The proportion_samples of this SelectionConfig.  # noqa: E501
        :rtype: float
        """
        return self._proportion_samples

    @proportion_samples.setter
    def proportion_samples(self, proportion_samples):
        """Sets the proportion_samples of this SelectionConfig.


        :param proportion_samples: The proportion_samples of this SelectionConfig.  # noqa: E501
        :type: float
        """

        self._proportion_samples = proportion_samples

    @property
    def strategies(self):
        """Gets the strategies of this SelectionConfig.  # noqa: E501


        :return: The strategies of this SelectionConfig.  # noqa: E501
        :rtype: list[SelectionConfigEntry]
        """
        return self._strategies

    @strategies.setter
    def strategies(self, strategies):
        """Sets the strategies of this SelectionConfig.


        :param strategies: The strategies of this SelectionConfig.  # noqa: E501
        :type: list[SelectionConfigEntry]
        """
        if self._configuration.client_side_validation and strategies is None:
            raise ValueError("Invalid value for `strategies`, must not be `None`")  # noqa: E501

        self._strategies = strategies

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(SelectionConfig, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SelectionConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SelectionConfig):
            return True

        return self.to_dict() != other.to_dict()
