""" The lightly.cli module provides a console interface
    for training self-supervised models, embedding,
    and filtering datasets
"""

# Copyright (c) 2020. Lightly AG and its affiliates.
# All Rights Reserved

from lightly.cli.lightly_cli import lightly_cli
from lightly.cli.train_cli import train_cli
from lightly.cli.embed_cli import embed_cli
from lightly.cli.upload_cli import upload_cli
from lightly.cli.download_cli import download_cli
from lightly.cli.crop_cli import crop_cli
