""" Collection of Active Learning Scorers """

# Copyright (c) 2020. Lightly AG and its affiliates.
# All Rights Reserved

from lightly.active_learning.scorers.scorer import Scorer
from lightly.active_learning.scorers.classification import ScorerClassification
from lightly.active_learning.scorers.detection import ScorerObjectDetection
from lightly.active_learning.scorers.semantic_segmentation import ScorerSemanticSegmentation
