""" Collection of Selection Configurations """

# Copyright (c) 2020. Lightly AG and its affiliates.
# All Rights Reserved

from lightly.active_learning.config.selection_config import SelectionConfig
