# coding: utf-8

"""
    Lightly API

    Lightly.ai enables you to do self-supervised learning in an easy and intuitive way. The lightly.ai OpenAPI spec defines how one can interact with our REST API to unleash the full potential of lightly.ai  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@lightly.ai
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from lightly.openapi_generated.swagger_client.configuration import Configuration


class DatasourceRawSamplesMetadataData(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'has_more': 'bool',
        'cursor': 'str',
        'data': 'list[DatasourceRawSamplesMetadataDataRow]'
    }

    attribute_map = {
        'has_more': 'hasMore',
        'cursor': 'cursor',
        'data': 'data'
    }

    def __init__(self, has_more=None, cursor=None, data=None, _configuration=None):  # noqa: E501
        """DatasourceRawSamplesMetadataData - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._has_more = None
        self._cursor = None
        self._data = None
        self.discriminator = None

        self.has_more = has_more
        self.cursor = cursor
        self.data = data

    @property
    def has_more(self):
        """Gets the has_more of this DatasourceRawSamplesMetadataData.  # noqa: E501

        Set to `false` if end of list is reached. Otherwise `true`.  # noqa: E501

        :return: The has_more of this DatasourceRawSamplesMetadataData.  # noqa: E501
        :rtype: bool
        """
        return self._has_more

    @has_more.setter
    def has_more(self, has_more):
        """Sets the has_more of this DatasourceRawSamplesMetadataData.

        Set to `false` if end of list is reached. Otherwise `true`.  # noqa: E501

        :param has_more: The has_more of this DatasourceRawSamplesMetadataData.  # noqa: E501
        :type: bool
        """
        if self._configuration.client_side_validation and has_more is None:
            raise ValueError("Invalid value for `has_more`, must not be `None`")  # noqa: E501

        self._has_more = has_more

    @property
    def cursor(self):
        """Gets the cursor of this DatasourceRawSamplesMetadataData.  # noqa: E501

        A cursor that indicates the current position in the list. Must be passed to future requests to continue reading from the same list.   # noqa: E501

        :return: The cursor of this DatasourceRawSamplesMetadataData.  # noqa: E501
        :rtype: str
        """
        return self._cursor

    @cursor.setter
    def cursor(self, cursor):
        """Sets the cursor of this DatasourceRawSamplesMetadataData.

        A cursor that indicates the current position in the list. Must be passed to future requests to continue reading from the same list.   # noqa: E501

        :param cursor: The cursor of this DatasourceRawSamplesMetadataData.  # noqa: E501
        :type: str
        """
        if self._configuration.client_side_validation and cursor is None:
            raise ValueError("Invalid value for `cursor`, must not be `None`")  # noqa: E501

        self._cursor = cursor

    @property
    def data(self):
        """Gets the data of this DatasourceRawSamplesMetadataData.  # noqa: E501

        Array containing the raw samples metadata objects  # noqa: E501

        :return: The data of this DatasourceRawSamplesMetadataData.  # noqa: E501
        :rtype: list[DatasourceRawSamplesMetadataDataRow]
        """
        return self._data

    @data.setter
    def data(self, data):
        """Sets the data of this DatasourceRawSamplesMetadataData.

        Array containing the raw samples metadata objects  # noqa: E501

        :param data: The data of this DatasourceRawSamplesMetadataData.  # noqa: E501
        :type: list[DatasourceRawSamplesMetadataDataRow]
        """
        if self._configuration.client_side_validation and data is None:
            raise ValueError("Invalid value for `data`, must not be `None`")  # noqa: E501

        self._data = data

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(DatasourceRawSamplesMetadataData, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DatasourceRawSamplesMetadataData):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, DatasourceRawSamplesMetadataData):
            return True

        return self.to_dict() != other.to_dict()
